/**
 * The copyrights, all other intellectual and industrial
 * property rights are retained by XMOS and/or its licensors.
 * Terms and conditions covering the use of this code can
 * be found in the Xmos End User License Agreement.
 *
 * Copyright XMOS Ltd 2014
 *
 * In the case where this code is a modification of existing code
 * under a separate license, the separate license terms are shown
 * below. The modifications to the code are still covered by the
 * copyright notice above.
 *
 **/

#include <platform.h>
#include <xs1.h>
#include <print.h>
#include <xscope.h>
#include "xtcp.h"
#include "web_server.h"
#include "ethernet_board_support.h"
#include <stdio.h>
#include <string.h>

/* Interfaces port definitions
 * These intializers are taken from the ethernet_board_support.h header for
 * XMOS dev boards. If you are using a different board you will need to
 * supply explicit port structure intializers for these values  */
ethernet_xtcp_ports_t xtcp_ports =
  {    on ETHERNET_DEFAULT_TILE:  OTP_PORTS_INITIALIZER,
       ETHERNET_DEFAULT_SMI_INIT,
       ETHERNET_DEFAULT_MII_INIT_lite,
       ETHERNET_DEFAULT_RESET_INTERFACE_INIT};

/* GPIO slice on triangle slot
 * PORT_4E connected to the 4 LEDs and PORT_8D connected to 2 buttons */
on tile[0]: port p_led=XS1_PORT_4E;
on tile[0]: port p_button=XS1_PORT_8D;


/* IP configuration.
 * Change this to suit your network.
 * Leave all as 0 values to use DHCP */
xtcp_ipconfig_t ipconfig = {
  { 0, 0, 0, 0 }, // IP address (eg 192,168,0,2)
  { 0, 0, 0, 0 }, // Netmask (eg 255,255,255,0)
  { 0, 0, 0, 0 }  // Gateway (eg 192,168,0,1)
};

/* Function to get 32-bit timer value as a string */
int get_timer_value(char buf[])
{
  /* Declare a timer resource */
  timer tmr;
  unsigned time;
  int len;
  /* Read the timer value in a variable */
  tmr :> time;
  /* Convert the timer value to string */
  sprintf(buf, "%u", time);
  return len;
}

/* Function to initialize the GPIO */
void init_gpio(void)
{
  /* Set all LEDs to OFF (Active low)*/
  p_led <: 0x0F;
}

/* Function to set LED state - ON/OFF */
void set_led_state(int led_id, int val)
{
  int value;
  /* Read port value into a variable */
  p_led :> value;
  if (!val) {
      p_led <: (value | (1 << led_id));
  } else {
      p_led <: (value & ~(1 << led_id));
  }
}

/* Function to read current button state */
int get_button_state(int button_id)
{
  int value;
  p_button :> value;
  value &= (1 << button_id);
  return (value >> button_id);
}

/* Function to handle the HTTP connections (TCP events)
 * from the TCP server task through 'c_xtcp' channels */
void http_handler(chanend c_xtcp) {

  xtcp_connection_t conn;  /* TCP connection information */

  /* Initialize webserver */
  web_server_init(c_xtcp, null, null);
  /* Initialize web application state */
  init_web_state();
  init_gpio();

  while (1) {
    select
      {
      case xtcp_event(c_xtcp,conn):
        /* Handles HTTP connections and other TCP events */
        web_server_handle_event(c_xtcp, null, null, conn);
        break;
      }
  }
}

/* The main starts two tasks (functions) in two different logical cores
 * and a channel is used for communication between the tasks */
int main(void) {
    /* Channel array declaration */
    chan c_xtcp[1];

    /* 'par' statement to run tasks (functions) in parallel */
    par
    {
          /* This function starts TCP/IP and Ethernet components in a separate core
           * but eventually uses two cores: one for Ethernet MII driver that
           * communicates to Ethernet PHY device and other for TCP/IP server
           */
          on ETHERNET_DEFAULT_TILE: ethernet_xtcp_server(xtcp_ports,
                                                         ipconfig,
                                                         c_xtcp,
                                                         1);
          /* This function runs in a separate core and handles the TCP events
           * i.e the HTTP connections from the above TCP server task
           * through the channel 'c_xtcp[0]'
           */
          on tile[0]: http_handler(c_xtcp[0]);

    }
    return 0;
}
